"""Consume a Sainsbury's webpage containing a list of groceries and return JSON
"""
from bs4 import BeautifulSoup
import urllib
import json


def get_product_urls(url):
    """Take the URL of the HTML page containing the list of groceries
    and return a generator of the urls associated with each available
    product.

    Parameters
    ----------
    url : string
         URL of web page. For example the development page URL is
         http://hiring-tests.s3-website-eu-west-1.amazonaws.com/2015_Developer_Scrape/sainsburys-apricot-ripe---ready-320g.html

    Returns
    -------
    A Python generator of urls (url1, url2, url3, ...)
    """
    soup = BeautifulSoup(urllib.urlopen(url))
    products = soup.find_all("div", class_="product")
    return (p.find("a")["href"] for p in products)

def process_url(url):
    """Process the url and return a dictionary containing the following keys:

    - size : the size of the html page (without assets) in kilobytes
      (1 kilobyte = 1000 bytes), with 1 decimal accuracy.
    - title : the name of the product.
    - unit_price : the unit price of the product in pounds.
    - description : the description of the product.

    Parameters
    ----------
    url : string
         A url corresponding to an individual product.

    Returns
    -------
         A Python dictionary containing the 'size', 'title',
         'unit_price' and 'description' of the product associated with
         the input url.
    """
    data = {}
    # Get the page size from the http request header
    page = urllib.urlopen(url)
    data['size'] = "%.1fkb" % (int(page.info().getheaders("Content-Length")[0]) / 1000.0) # CHECK
    # Get the title, unit_price and description
    psoup = BeautifulSoup(page)
    data['title'] = psoup.h1.string
    data['unit_price'] = float(psoup.find("p", class_="pricePerUnit").contents[0][2:])
    data['description'] = psoup.find("div", class_="productText").getText().strip()
    return data

def tojson(results):
    """Export input results list of product dictionaries into JSON.

    The function exports the results list to JSON and defines 'results' as a key of the JSON output.
    The function also computes the total prices of all products in the
    results list and defines it as a separate key in the JSON
    output.

    Parameters
    ----------
    results : list of dicts
         List of dictionaries describing each product. See process_url.

    Returns
    -------
    A JSON string describing the products, including:
    - a 'results' list of all individual products, with their 'description', 'size', 'title' and 'unit_price'.
    - a 'total' floating point number that sums the total price of all the products on the page.

    Example output

    {
        "results": [
            {
                "description": "Great to eat now - refrigerate at home 1 of 5 a day 1 avocado counts as 1 of your 5...",
                "size": "90.6kb",
                "title": "Sainsbury's Avocado, Ripe & Ready x2",
                "unit_price": 1.8
            },
            {
                "description": "Great to eat now - refrigerate at home 1 of 5 a day 1...",
                "size": "87kb",
                "title": "Sainsbury's Avocado, Ripe & Ready x4",
                "unit_price": 2.0
            }
        ],
        "total": 3.8
    }
    """
    data = {}
    data["results"] = results
    data["total"] = sum([p['unit_price']  for p in results])
    return json.dumps(data, sort_keys=True, indent=4, separators=(',', ': '))

def scrape(url,
           get_product_urls=get_product_urls,
           process_url=process_url,
           tojson=tojson):
    """Scrape the individual URL and return the JSON data

    Parameters
    ----------
    url : string
         The url of the page to be scraped

    get_product_urls : callable
         A function that returns a generator of the product urls.

    process_url : callable
         A function that processes the result urls and returns the Python results dict.

    tojson : callable
         A function that transforms the list of results into a JSON string.
    """
    urls = get_product_urls(url)  # a generator
    results = map(process_url, urls)
    return tojson(results)

if __name__ == "__main__":
    url = "http://hiring-tests.s3-website-eu-west-1.amazonaws.com/2015_Developer_Scrape/5_products.html"
    print scrape(url)
